/*  This file is a part of Dict. Dict is a Series 60 application for browsing
	dict databases (http:\\www.dict.org) stored in the phone's file system.
	Copyright (C) 2005, Kostas Giannakakis
	Web page: http://users.forthnet.gr/ath/kgiannak/

	Dict is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Dict is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Dict; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __FILE_RANDOM_READER_H__
#define __FILE_RANDOM_READER_H__

#include <e32std.h>
#include <e32def.h>
#include <f32file.h>

#define KBufferSize		256

/** FileRandomReader class wraps up an RFile object to facilitate the reading
	of a file. The class exposes methods for reading basic types directly,
	from any position at the file. This minimises the lines
	of the code needed by the caller of the class, since there is no need of
	seeking to the desired location, error checking and type conversion. The class
	will return dummy data when a file operation error occurs. It is responsibility
	of the caller to test for the integrity of the data. FileRandomReader class
	uses a buffer mechanism - it always read a big chunk of memory around the
	requested location. Subsequent read requests can be served from this buffer,
	as long as the requested locations are close to the location of the first read.
	This could make reading more efficient.
*/
class FileRandomReader
{
public:
	/** Constructor. */
	FileRandomReader();

	/** Destructor. */
	~FileRandomReader();

	/** Opens the file handle.
		@param aFileName The name of the file to open.
		@param aReadBufferInBothDirections If true, the internal buffer
			   grows in both directions from the location of the first
			   read. Defaults to false.
		@result The code returned from opening the RFile object.
	*/
	TInt Open(const TDesC& aFileName,
			  TBool aReadBufferInBothDirections = EFalse);

	/** Closes the file handle. */
	void Close();

	/** Returns the size of the file.
		@result The file size.*/
	TInt GetSize();

	/** Returns a reference to the original RFile handle.
		@result A reference to the original RFile handle.*/
	RFile &GetFileHandle() { return iFile;};

	/** Reads a byte array from a specific location at the file.
		@param aDes Buffer to place the read characters. OUT parameter.
		@param aPos Position of the file to start the reading.
		@param aLength Number of bytes to read.
		@result True if the operation is successful..
	*/
	TBool Read(TDes8& aDes, TInt aPos, TInt aLength);

	/** Reads a byte from a specific location at the file.
		@param aPos Position of the file to read the byte from.
		@result The value read.
	*/
	TUint8 ReadByte(TInt aPos);

	/** Reads an unsigned 16-bit integer from a specific location at the file.
		@param aPos Position of the file to read the integer from.
		@result The value read.
	*/
	TUint16 ReadWord(TInt aPos);

	/** Reads an integer from a specific location at the file.
		@param aPos Position of the file to read the integer from.
		@result The value read.
	*/
	TInt ReadInt(TInt aPos);

private:

	/** Updates the internal buffer.
		@param aPos Position to start reading.
		@param aLength Number of bytes to read.
		@result True if the operation is successful.*/
	TBool UpdateBuffer(TInt aPos, TInt aLength);

private:
	/** File handle. */
	RFile iFile;

	/** Boolean variable that is True if the file is open.*/
	TBool iIsOpen;

	/** File length. */
	TInt iLength;

	/** Boolean variable that is True when the buffer expands in both
		directions. Defaults to False.*/
	TBool iReadBufferInBothDirections;

	/** Buffer used for storing a big chunk of memory read from the file. */
	TBuf8<KBufferSize> iBuffer;

	/** File position of the first byte that was read into the buffer. */
	TInt iBufferStart;

	/** File position of the last byte that was read into the buffer. */
	TInt iBufferEnd;
};

#endif //__FILE_RANDOM_READER_H__